// Pt1BaseTuner.cpp: PT1`[iNX
//
/////////////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "BonSDK.h"
#include "Pt1BaseTuner.h"
#include <TChar.h>


/////////////////////////////////////////////////////////////////////////////
// t@C[J萔ݒ
/////////////////////////////////////////////////////////////////////////////

// foCX
#define DEVICENAME_TER	TEXT("Earthsoft PT1 ISDB-T fW^`[i")
#define DEVICENAME_SAT	TEXT("Earthsoft PT1 ISDB-S fW^`[i")

// obt@ݒ
#define FIFOBUFFTIME	2UL														// obt@ = 2b
#define FIFOBLOCKSIZE	(CPt1Board::DMABLK_SIZE / CPt1Board::DMACHK_UNIT / 3UL)	// ubNTCY
#define FIFOBUFFNUM		(0x3C0000UL / FIFOBLOCKSIZE * FIFOBUFFTIME)				// 30MbpsƂ


/////////////////////////////////////////////////////////////////////////////
// PT1`[ivpeBNX
/////////////////////////////////////////////////////////////////////////////

const bool CPt1BaseTunerProperty::LoadProperty(IConfigStorage *pStorage)
{
	if(!pStorage)return false;

	// ݒǂݏo
	m_bDefaultLnbPower = pStorage->GetBoolItem(TEXT("DefaultLnbPower"	), false);
	m_bEnaleVHF		   = pStorage->GetBoolItem(TEXT("EnaleVHF"			), true	);
	m_bEnaleCATV	   = pStorage->GetBoolItem(TEXT("EnaleCATV"			), true	);
	m_bEnaleCS		   = pStorage->GetBoolItem(TEXT("EnaleCS"			), true	);
	m_bEnaleBS		   = pStorage->GetBoolItem(TEXT("EnaleBS"			), true	);

	return true;
}

const bool CPt1BaseTunerProperty::SaveProperty(IConfigStorage *pStorage)
{
	if(!pStorage)return false;
	
	// ݒۑ
	try{
		if(!pStorage->SetBoolItem(TEXT("DefaultLnbPower"), m_bDefaultLnbPower	))throw ::BON_EXPECTION();
		if(!pStorage->SetBoolItem(TEXT("EnaleVHF"		), m_bEnaleVHF			))throw ::BON_EXPECTION();
		if(!pStorage->SetBoolItem(TEXT("EnaleCATV"		), m_bEnaleCATV			))throw ::BON_EXPECTION();
		if(!pStorage->SetBoolItem(TEXT("EnaleCS"		), m_bEnaleCS			))throw ::BON_EXPECTION();
		if(!pStorage->SetBoolItem(TEXT("EnaleBS"		), m_bEnaleBS			))throw ::BON_EXPECTION();
		}
	catch(CBonException &Exception){
		// G[
		Exception.Notify();
		return false;
		}
	
	return true;
}

IConfigTarget * CPt1BaseTunerProperty::GetConfigTarget(void)
{
	// RtBO^[QbgԂ
	return m_pConfigTarget;
}

const bool CPt1BaseTunerProperty::CopyProperty(const IConfigProperty *pProperty)
{
	const IPt1BaseTunerProperty *pPt1BaseTunerProperty = dynamic_cast<const IPt1BaseTunerProperty *>(pProperty);
	if(!pPt1BaseTunerProperty)return false;
	
	// vpeBRs[
	*(static_cast<IPt1BaseTunerProperty *>(this)) = *pPt1BaseTunerProperty;
	
	return true;
}

const CBonString CPt1BaseTunerProperty::GetDialogClassName()
{
	// vpeB_CAO͔
	return CBonString::GetEmpty();
}

CPt1BaseTunerProperty::CPt1BaseTunerProperty(IBonObject *pOwner)
	: CBonObject(pOwner)
	, m_pConfigTarget(dynamic_cast<IConfigTarget *>(pOwner))
{
	// ftHglݒ
	m_bDefaultLnbPower = false;
	m_bEnaleVHF = true;
	m_bEnaleCATV = true;
	m_bEnaleCS = true;
	m_bEnaleBS = true;
}

CPt1BaseTunerProperty::~CPt1BaseTunerProperty(void)
{
	// Ȃ
}


/////////////////////////////////////////////////////////////////////////////
// PT1`[iNX
/////////////////////////////////////////////////////////////////////////////

CPt1Manager *CPt1BaseTuner::m_pPt1Manager = NULL;
DWORD CPt1BaseTuner::m_dwInstanceCount = 0UL;

const BONGUID CPt1BaseTuner::GetDeviceType(void) const
{
	// foCX̃^CvԂ
	return ::BON_NAME_TO_GUID(TEXT("IHalTsTuner"));
}

const CBonString CPt1BaseTuner::GetDeviceName() const
{
	static const CBonString deviceName[] =
	{
		CBonString(DEVICENAME_TER), CBonString(DEVICENAME_SAT)
	};

	// foCXԂ
	return deviceName[m_dwTunerType];
}

const DWORD CPt1BaseTuner::GetTotalDeviceNum(void) const
{
	// `[ȋԂ
	return m_pPt1Manager->GetTotalTunerNum(m_dwTunerType);
}

const DWORD CPt1BaseTuner::GetActiveDeviceNum(void) const
{
	// gp̃`[iԂ
	return m_pPt1Manager->GetActiveTunerNum(m_dwTunerType);
}

const DWORD CPt1BaseTuner::OpenTuner(void)
{
	// UN[Y
	CloseTuner();
	
	try{
		// `[iI[v
		if(!(m_pTuner = m_pPt1Manager->OpenTuner(m_dwTunerType)))throw ::BON_EXPECTION(TEXT("`[iI[vs"));

		// LNBdݒ
		if(m_dwTunerType == Device::ISDB_S){
			if(!SetLnbPower(m_Property.m_bDefaultLnbPower))throw ::BON_EXPECTION(TEXT("LNBdݒ莸s"));
			}

		// ftHg`lݒ
		SetChannel(m_dwCurSpace, m_dwCurChannel);

		// Xg[MJn
		if(!m_pTuner->StartStream(this))throw ::BON_EXPECTION(TEXT("TSXg[MJns"));
		}
	catch(CBonException &Exception){
		// G[
		Exception.Notify();
		CloseTuner();
		return EC_OPEN_FAILED;
		}
	
	return EC_OPEN_SUCCESS;
}

void CPt1BaseTuner::CloseTuner(void)
{
	// `[iN[Y
	if(m_pTuner){
		m_pTuner->StopStream();
		m_pTuner->CloseTuner();
		m_pTuner = NULL;
		}
}

const CBonString CPt1BaseTuner::EnumTuningSpace(const DWORD dwSpace) const
{
	// gp\ȃ`[jOԂԂ
	if(dwSpace >= m_TuningSpace.size())return CBonString::GetEmpty();

	// `[jOԖԂ
	return CBonString(m_TuningSpace[dwSpace]->szSpaceName);
}

const CBonString CPt1BaseTuner::EnumChannelName(const DWORD dwSpace, const DWORD dwChannel) const
{
	// gp\ȃ`lԂ
	if(dwSpace >= m_TuningSpace.size())return CBonString::GetEmpty();

	if(dwChannel >= m_TuningSpace[dwSpace]->dwChNum)return CBonString::GetEmpty();

	// `lԂ
	return CBonString(m_TuningSpace[dwSpace]->pChConfig[dwChannel].szChName);
}

const DWORD CPt1BaseTuner::GetDeviceID(void) const
{
	return m_pTuner->GetTunerID();
}

const DWORD CPt1BaseTuner::GetCurSpace(void) const
{
	// ݂̃`[jOԂԂ
	return m_dwCurSpace;
}

const DWORD CPt1BaseTuner::GetCurChannel(void) const
{
	// ݂̃`lԂ
	return m_dwCurChannel;
}

const bool CPt1BaseTuner::SetChannel(const DWORD dwSpace, const DWORD dwChannel)
{
	// ȂAhNXŃI[o[ChȂ΂ȂȂ
	::BON_ASSERT(false);
	
	return false;
}

const bool CPt1BaseTuner::SetLnbPower(const bool bEnable)
{
	// LNBd
	return (m_pTuner)? m_pTuner->SetLnbPower(bEnable) : false;
}

const float CPt1BaseTuner::GetSignalLevel(void) const
{
	// CNR擾
	return (m_pTuner)? m_pTuner->GetSignalLevel() : 0.0f;
}

const bool CPt1BaseTuner::SetProperty(const IConfigProperty *pProperty)
{
	if(!pProperty)return false;

	// vpeBݒ
	return m_Property.CopyProperty(pProperty);
}

const bool CPt1BaseTuner::GetProperty(IConfigProperty *pProperty)
{
	if(!pProperty)return false;

	// vpeB擾
	return pProperty->CopyProperty(&m_Property);
}

const CBonString CPt1BaseTuner::GetPropertyClassName()
{
	static const CBonString propertyClassName(TEXT("CPt1BaseTunerProperty"));

	// vpeBNX擾
	return propertyClassName;
}

const bool CPt1BaseTuner::SetOutputDecoder(IMediaDecoder *pDecoder)
{
	m_pOutputDecoder = pDecoder;
	return true;
}

CPt1BaseTuner::CPt1BaseTuner(IBonObject *pOwner, const Device::ISDB dwTunerType)
	: CBonObject(pOwner)
	, m_Property(static_cast<IConfigTarget *>(this))
	, m_pTuner(NULL)
	, m_dwTunerType(dwTunerType)
	, m_dwCurSpace(0UL)
	, m_dwCurChannel(0UL)
	, m_pOutputDecoder(0)
{
	// }l[WCX^X
	if(!m_dwInstanceCount++){
		m_pPt1Manager = new CPt1Manager;
		}

	::BON_ASSERT(m_pPt1Manager != NULL);

	// ftHgvpeBf
	SetProperty(&m_Property);
}

CPt1BaseTuner::~CPt1BaseTuner(void)
{
	CloseTuner();
	
	// }l[WCX^XJ
	if(!--m_dwInstanceCount){
		BON_SAFE_DELETE(m_pPt1Manager);
		}
}

void CPt1BaseTuner::OnTsStream(CPt1Tuner *pTuner, const BYTE *pData, const DWORD dwSize)
{
	if (m_pOutputDecoder != 0)
	{
		m_pOutputDecoder->InputMedia(pData, dwSize);
	}
}
