// Pt1BaseTuner.h: PT1`[iNX
//
/////////////////////////////////////////////////////////////////////////////

#pragma once


#include "Pt1Device.h"
#include "Pt1DriverIf.h"
#include <vector>
#include <queue>


/////////////////////////////////////////////////////////////////////////////
// PT1`[ivpeBNX
/////////////////////////////////////////////////////////////////////////////

class CPt1BaseTunerProperty	:	public CBonObject,
								public IConfigProperty,
								public IPt1BaseTunerProperty
{
public:
// CBonObject
	DECLARE_IBONOBJECT(CPt1BaseTunerProperty)

// IConfigProperty
	virtual const bool LoadProperty(IConfigStorage *pStorage);
	virtual const bool SaveProperty(IConfigStorage *pStorage);
	virtual const bool CopyProperty(const IConfigProperty *pProperty);
	virtual IConfigTarget * GetConfigTarget(void);
	virtual const CBonString GetDialogClassName();

// CPt1BaseTunerProperty
	CPt1BaseTunerProperty(IBonObject *pOwner);
	virtual ~CPt1BaseTunerProperty(void);
	
protected:
	IConfigTarget *m_pConfigTarget;
};


/////////////////////////////////////////////////////////////////////////////
// PT1`[iNX
/////////////////////////////////////////////////////////////////////////////

class CPt1BaseTuner :	public CBonObject,
						public IHalDevice,
						public IHalTsTuner,
						public IConfigTarget,
						protected CPt1Tuner::ITsReceiver
{
public:
// CBonObject
	DECLARE_IBONOBJECT(CPt1BaseTuner)

// IHalDevice
	virtual const BONGUID GetDeviceType(void) const;
	virtual const CBonString GetDeviceName() const;
	virtual const DWORD GetTotalDeviceNum(void) const;
	virtual const DWORD GetActiveDeviceNum(void) const;

// IHalTsTuner
	virtual const DWORD OpenTuner(void);
	virtual void CloseTuner(void);
	
	virtual const CBonString EnumTuningSpace(const DWORD dwSpace) const;
	virtual const CBonString EnumChannelName(const DWORD dwSpace, const DWORD dwChannel) const;
	
	virtual const DWORD GetDeviceID(void) const;
	virtual const DWORD GetCurSpace(void) const;
	virtual const DWORD GetCurChannel(void) const;

	virtual const bool SetChannel(const DWORD dwSpace, const DWORD dwChannel);
	virtual const bool SetLnbPower(const bool bEnable);
	virtual const float GetSignalLevel(void) const;

	virtual const bool SetOutputDecoder(IMediaDecoder *pDecoder);

// IConfigTarget
	virtual const bool SetProperty(const IConfigProperty *pProperty);
	virtual const bool GetProperty(IConfigProperty *pProperty);
	virtual const CBonString GetPropertyClassName();

// CPt1BaseTuner
	CPt1BaseTuner(IBonObject *pOwner, const Device::ISDB dwTunerType = Device::ISDB_T);
	virtual ~CPt1BaseTuner(void);

	struct PT_CH_CONFIG
	{
		const TCHAR szChName[32];		// `l
		const DWORD dwChIndex;			// PT1`lCfbNX
		const WORD wTsID;				// TS ID
	};

	struct PT_CH_SPACE
	{
		const TCHAR szSpaceName[32];	// Ԗ
		const PT_CH_CONFIG *pChConfig;	// `lݒ
		const DWORD dwChNum;			// `l
	};

protected:
// CPt1Tuner::ITsReceiver
	virtual void OnTsStream(CPt1Tuner *pTuner, const BYTE *pData, const DWORD dwSize);

	static CPt1Manager *m_pPt1Manager;
	static DWORD m_dwInstanceCount;
	IMediaDecoder *m_pOutputDecoder;

	CPt1BaseTunerProperty m_Property;

	CPt1Tuner *m_pTuner;

	const Device::ISDB m_dwTunerType;
	std::vector<const PT_CH_SPACE *> m_TuningSpace;

	DWORD m_dwCurSpace;
	DWORD m_dwCurChannel;
};
