// Pt1Device.h: PT1foCXNX
//
/////////////////////////////////////////////////////////////////////////////

#pragma once


#include "EARTH_PT.h"
#include "OS_Library.h"
#include <Vector>
#include <Queue>


using namespace EARTH;
using namespace OS;
using namespace PT;
using namespace std;


/////////////////////////////////////////////////////////////////////////////
// PT}l[WNX
/////////////////////////////////////////////////////////////////////////////

class CPt1Board;
class CPt1Tuner;

class CPt1Manager
{
public:
	CPt1Manager(void);
	virtual ~CPt1Manager(void);

	const DWORD GetTotalTunerNum(const Device::ISDB dwTunerType);
	const DWORD GetActiveTunerNum(const Device::ISDB dwTunerType);

	CPt1Tuner * OpenTuner(const Device::ISDB dwTunerType);

protected:
	const bool EnumerateBoards(void);
	const bool ReleaseInstance(void);
	
	Bus *m_pPtBus;
	vector<CPt1Board *> m_Boards;

	Library *m_pPtLibrary;
};


/////////////////////////////////////////////////////////////////////////////
// PT1{[hNX
/////////////////////////////////////////////////////////////////////////////

class CPt1Board
{
friend CPt1Tuner;

public:
	enum{TUNER_1ST = 0UL, TUNER_2ND = 1UL, TUNER_NUM = 2UL};

	enum{
		DMABUF_SIZE = 6UL,									// DMAubN
		DMABUF_NUM  = 1UL,									// DMAobt@
		DMABLK_SIZE = 4096UL * Device::BUFFER_PAGE_COUNT,	// DMAubNTCY
		DMACHK_UNIT = 1UL,									// DMA]`FbN
		DMACHK_WAIT = 5UL,									// DMA]`FbNEFCg
		DMACHK_TIMEOUT = 1000UL								// DMA]^CAEg
		};

	CPt1Board(const DWORD dwBoardID, Device *pDevice, const Bus::DeviceInfo *pDeviceInfo);
	virtual ~CPt1Board(void);

	CPt1Tuner * OpenTuner(const Device::ISDB dwTunerType);
	void CloseTuner(CPt1Tuner *pTuner);

	const DWORD GetActiveTunerNum(const Device::ISDB dwTunerType);

protected:
	const bool StartStream(CPt1Tuner *pTuner);
	void StopStream(CPt1Tuner *pTuner);

	const bool SetLnbPower(CPt1Tuner *pTuner, const bool bEnable);

	const bool IsValidTuner(const CPt1Tuner *pTuner) const;
	const DWORD GetOpenTunerNum(void) const;
	const DWORD GetStreamTunerNum(void) const;
	const DWORD GetLnbOnTunerNum(void) const;

	struct DISPATCH_BUFF
	{
		DWORD dwPacketPos;
		BYTE *pDataPtr;
		BYTE abyDataBuff[DMABLK_SIZE * Device::ISDB_COUNT * TUNER_NUM];
	};
	
	const DWORD m_dwBoardID;
	Device *m_pDevice;
	Bus::DeviceInfo m_DeviceInfo;
	
	vector<CPt1Tuner *> m_TunerList;
	DISPATCH_BUFF m_aDispatchBuff[TUNER_NUM * Device::ISDB_COUNT];

	CSmartMutex m_BoardLock;
	CSmartLock m_TunerLock;

private:
	const bool OpenBoard(void);
	void CloseBoard(void);

	void StreamingThread(CSmartThread<CPt1Board> &thread, const ISignal &bKillSignal, const ULONG_PTR param);
	void DispatchStream(const DWORD *pdwData, const DWORD dwNum);

	CSmartThread<CPt1Board> m_StreamingThread;
};


/////////////////////////////////////////////////////////////////////////////
// PT1`[iNX
/////////////////////////////////////////////////////////////////////////////

class CPt1Tuner
{
friend CPt1Board;

public:
	class ITsReceiver :	public IBonObject
	{
	public:
		virtual void OnTsStream(CPt1Tuner *pTuner, const BYTE *pData, const DWORD dwSize) = 0;
	};

	CPt1Tuner(CPt1Board &Board, const DWORD dwTunerID, const Device::ISDB dwTunerType);
	virtual ~CPt1Tuner();

	void CloseTuner(void);

	const bool StartStream(ITsReceiver *pTsReceiver);
	void StopStream(void);

	const DWORD GetTunerID(void);
	const bool SetChannel(const DWORD dwChannel, const DWORD dwOffset = 0UL);
	const bool SetTsID(const WORD wTsID);
	const bool SetLnbPower(const bool bEnable);
	const float GetSignalLevel(void);

protected:
	void OnTsStream(const BYTE *pData, const DWORD dwSize);

	CPt1Board &m_Board;

	const DWORD m_dwTunerID;
	const Device::ISDB m_dwTunerType;

	bool m_bIsOpen;
	bool m_bIsStream;
	bool m_bIsLnbOn;
	
	ITsReceiver *m_pTsReceiver;
	CSmartLock m_Lock;
};
